"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.UserBridge = exports.resolveNick = exports.defaultConfig = exports.messages = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _events = require("events");

var _debug = require("debug");

var _debug2 = _interopRequireDefault(_debug);

var _lodash = require("lodash");

var _escapeStringRegexp = require("escape-string-regexp");

var _escapeStringRegexp2 = _interopRequireDefault(_escapeStringRegexp);

var _irc = require("./irc");

var _irc2 = _interopRequireDefault(_irc);

var _telegram = require("./telegram");

var _telegram2 = _interopRequireDefault(_telegram);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Bridge module
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * @TODO: Fix problem with final nick already exist on IRC channel
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                */

var debug = (0, _debug2.default)("bridge");

var messages = exports.messages = {
    join: "[IRC/{nick} * joined to the channel *]"
};

var defaultConfig = exports.defaultConfig = {
    irc: null,
    telegram: null,
    lang: null,
    oneConnectionByUser: false,
    showJoinLeft: true,
    prefix: "telegram_",
    suffix: "",
    ircScapeCharacter: ""
};

var userInstances = [];

/**
 * Resolve nicks with the prefix and suffix options
 * @param {object} userData
 * @param {object} options
 * @return {string}
 */
var resolveNick = exports.resolveNick = function resolveNick(userData, options) {
    "use strict";

    var name = (userData.username || userData.first_name + "-" + userData.id).replace(/[^a-z_]+/ig, '');
    var prefix = options.prefix,
        suffix = options.suffix;

    return "" + prefix + name + suffix;
};

/**
 * User bridge, for oneConnectionByUser option
 */

var UserBridge = exports.UserBridge = function (_EventEmitter) {
    _inherits(UserBridge, _EventEmitter);

    /**
     * Create new UserBridge
     * @param {object} userData The telegram format user information
     * @param {object} options Bridge options
     */
    function UserBridge(userData, options) {
        _classCallCheck(this, UserBridge);

        var _this = _possibleConstructorReturn(this, (UserBridge.__proto__ || Object.getPrototypeOf(UserBridge)).call(this));

        _this._options = (0, _lodash.assignIn)({}, {
            prefix: "tele_",
            suffix: ""
        }, options);

        _this._userData = userData;

        _this._name = userData.username || userData.first_name + "-" + userData.id;

        var rnick = resolveNick(userData, _this._options);

        var serveropts = (0, _lodash.assignIn)({}, options.irc, { nick: rnick });

        debug("Search irc connection for " + rnick + "@" + serveropts.server);

        _this._ircConnection = _irc2.default.getByServerOptions(serveropts);

        userInstances.push(_this);

        return _this;
    }

    /**
     * Get channel from the user bridge
     * @param {string} channelName IRC channel name
     * @return {IRCChannel}
     */


    _createClass(UserBridge, [{
        key: "getChannel",
        value: function getChannel(channelName) {
            var channel = this._ircConnection.getChannel(channelName);

            if (!channel) {
                channel = this._ircConnection.addChannel(channelName);
            }

            return channel;
        }

        /**
         * Final nickname (applied prefix/suffix);
         * @property nick
         */

    }, {
        key: "nick",
        get: function get() {
            return resolveNick(this._userData, this._options);
        }

        /**
         * Original name (telegram username)
         * @property name
         */

    }, {
        key: "name",
        get: function get() {
            return this._name;
        }

        /**
         * Original telegram id
         * @property tid
         */

    }, {
        key: "tid",
        get: function get() {
            return this._userData.id;
        }

        /**
         * Unique identifier, generated with username and server options
         * @property identifier
         */

    }, {
        key: "identifier",
        get: function get() {
            var tid = this.tid;
            var ircIdent = this._ircConnection.identifier;
            return tid + "@@" + ircIdent;
        }

        /**
         * Get UserBridge by options
         * @param {object} userData Original information
         * @param {object} options Bridge options
         * @return {UserBridge}
         */

    }], [{
        key: "getByOptions",
        value: function getByOptions(userData, options) {
            var rnick = resolveNick(userData, options);
            var ircOptions = (0, _lodash.assignIn)({}, options.irc, { nick: rnick });
            var irc = _irc2.default.getByServerOptions(ircOptions);
            var ircIdent = irc.identifier;
            var userIdent = userData.id + "@@" + ircIdent;
            debug("Find for UserBridge " + userIdent);
            var instance = userInstances.find(function (u) {
                return u.identifier === userIdent;
            });

            if (!instance) {
                debug("Not foud, creating");
                instance = new UserBridge(userData, options);
            }

            return instance;
        }
    }]);

    return UserBridge;
}(_events.EventEmitter);

/**
 * Bridge class
 */


var Bridge = function (_EventEmitter2) {
    _inherits(Bridge, _EventEmitter2);

    /**
     * Create new bridge
     * @param {string} name The name of the bridge
     * @param {object} options Options
     * @param {object} irc IRC options
     * @param {object} telegram Telegram options
     * @param {string} [lang] Language of bridge messages
     * @param {bool} [oneConnectionByUser] Create one new connection to IRC by
     *                                     each new user in telegram. By    
     *                                     default is false
     * @param {string} [prefix] User nickname prefix
     * @param {string} [suffix] User nickname suffix
     */
    function Bridge(name, ircChannel, telegramChannel, options) {
        _classCallCheck(this, Bridge);

        var _this2 = _possibleConstructorReturn(this, (Bridge.__proto__ || Object.getPrototypeOf(Bridge)).call(this));

        _this2._options = (0, _lodash.assignIn)({}, defaultConfig, options);

        _this2._ircUsers = [];

        debug("Create bridge " + name);

        _this2._ircConnector = _irc2.default.getByServerOptions(_this2._options.irc);

        _this2._ircChannel = _this2._ircConnector.addChannel(ircChannel);

        _this2._telegramConnector = _telegram2.default.getByToken(_this2._options.telegram.token, _this2._options.telegram);

        _this2._telegramChannel = _this2._telegramConnector.followChannel(telegramChannel);

        _this2._telegramSuccessMe = false;
        _this2._ircSuccessRegistered = false;

        _this2._handlers = {
            ircMessage: _this2._handleIRCMessage.bind(_this2),
            ircAction: _this2._handleIRCAction.bind(_this2),
            ircJoin: _this2._handleIRCJoin.bind(_this2),
            ircLeft: _this2._handleIRCLeft.bind(_this2),
            ircTopic: _this2._handleIRCTopic.bind(_this2),
            telegramMessage: _this2._handleTelegramMessage.bind(_this2),
            telegramJoin: _this2._handleTelegramJoin.bind(_this2),
            telegramLeft: _this2._handleTelegramLeft.bind(_this2)
        };

        _this2.bind();

        return _this2;
    }

    /**
     * Translate IRC nicks to Telegram nicks
     * @param {string} message Message to translate nicks
     * @return {string} Final message
     */


    _createClass(Bridge, [{
        key: "_translateIrcNicks",
        value: function _translateIrcNicks(message) {
            return this._ircUsers.map(function (x) {
                var name = x.name,
                    nick = x.nick;

                return { name: name, nick: nick };
            }).reduce(function (message, replaces) {
                var name = replaces.name,
                    nick = replaces.nick;

                var snick = (0, _escapeStringRegexp2.default)(nick);
                var regexptxt = "\\b" + snick + "\\b";
                var regexp = new RegExp(regexptxt, "g");
                return message.replace(regexp, "@" + name);
            }, message);
        }

        /**
         * Get UserBridge from a telegram user
         * @param {object} userData Telegram user information
         * @return {UserBridge}
         */

    }, {
        key: "_getIrcUser",
        value: function _getIrcUser(userData) {
            var user = this._ircUsers.find(function (x) {
                return x.tid === userData.id;
            });

            if (!user) {
                user = UserBridge.getByOptions(userData, this._options);
                this._ircUsers.push(user);
            }

            return user;
        }

        /**
         * Return true if there are a UserBridge with the passed irc nick
         * @param {string} nick IRC nick
         * @return {bool}
         */

    }, {
        key: "_haveIrcUser",
        value: function _haveIrcUser(nick) {
            debug("Check if have user " + nick);

            if (nick === this._ircConnector.nick) {
                return true;
            }

            return !!this._ircUsers.find(function (x) {
                return x.nick === nick;
            });
        }

        /**
         * Return the final IRCChannel from an UserBridge
         * @param {object} userData Telegram user information
         * @return {IRCChannel}
         */

    }, {
        key: "_getIrcUserChan",
        value: function _getIrcUserChan(userData) {
            var user = this._getIrcUser(userData);
            var chan = user.getChannel(this._ircChannel.name);
            return chan;
        }

        /**
         * Remove an user from IRC users list
         * @param {UserBridge} user UserBridge insntance
         * @return {UserBridge}
         */

    }, {
        key: "_removeIrcUser",
        value: function _removeIrcUser(user) {
            this._ircUsers = this._ircUsers.filter(function (x) {
                return x.identifier !== user.identifier;
            });
            return user;
        }

        /**
         * Handle incomming IRC message
         * @param {string} user IRC user
         * @param {string} message Message
         */

    }, {
        key: "_handleIRCMessage",
        value: function _handleIRCMessage(user, message) {
            if (this._haveIrcUser(user)) {
                return;
            }
            debug("irc in message", user);

            if (this._options.oneConnectionByUser) {
                message = this._translateIrcNicks(message);
            }

            var msg = "*<" + user + ">* " + message;
            this._telegramChannel.sendMessage(msg, { "parse_mode": 'markdown' });
        }

        /**
         * Handle incomming IRC action 
         * @param {string} user IRC user
         * @param {string} message Message
         */

    }, {
        key: "_handleIRCAction",
        value: function _handleIRCAction(user, message) {
            if (this._haveIrcUser(user)) {
                return;
            }
            debug("irc in action", user);

            if (this._options.oneConnectionByUser) {
                message = this._translateIrcNicks(message);
            }

            var msg = "_*<" + user + "> " + message + "*_";
            this._telegramChannel.sendMessage(msg, { "parse_mode": 'markdown' });
        }

        /**
         * Handle incomming IRC join message
         * @param {string} user IRC user
         */

    }, {
        key: "_handleIRCJoin",
        value: function _handleIRCJoin(user) {
            if (this._haveIrcUser(user)) {
                return;
            }
            debug("irc join", user);

            if (this._options.showJoinLeft) {
                var msg = "_*<" + user + "> has joined*_";
                this._telegramChannel.sendMessage(msg, { "parse_mode": 'markdown' });
            }
        }

        /**
         * Handle IRC part message
         * @param {string} user Irc user
         */

    }, {
        key: "_handleIRCLeft",
        value: function _handleIRCLeft(user) {
            if (this._haveIrcUser(user)) {
                return;
            }

            debug("irc left", user);

            if (this._options.showJoinLeft) {
                var msg = "_*" + user + " left the channel*_";
                this._telegramChannel.sendMessage(msg, { "parse_mode": 'markdown' });
            }
        }

        /**
         * Handle IRC topic message
         * @param {string} user Irc user
         */

    }, {
        key: "_handleIRCTopic",
        value: function _handleIRCTopic(channel, topic, nick) {
            if (this._haveIrcUser(nick)) {
                return;
            }
            debug("irc topic", topic, nick);
            var msg = "_*<" + nick + "> changed the topic to: " + topic + "*_";
            this._telegramChannel.sendMessage(msg);
        }

        /**
         * Handle Telegram incomming message
         * @param {object} user Telegram user data 
         * @param {string} message Text message
         */

    }, {
        key: "_handleTelegramMessage",
        value: function _handleTelegramMessage(user, message) {
            debug("telegram in message", user);

            var scapeChar = this._options.ircScapeCharacter;

            if (this._options.oneConnectionByUser) {
                var chan = this._getIrcUserChan(user);
                chan.sendMessage(message);
            } else if (scapeChar && message.startsWith(scapeChar)) {
                this._ircChannel.sendMessage(message);
            } else {
                var msg = "[Telegram/@" + user.username + "] " + message;
                this._ircChannel.sendMessage(msg);
            }
        }

        /**
         * Handle Telegram join message
         * @param {object} user Telegram user data
         */

    }, {
        key: "_handleTelegramJoin",
        value: function _handleTelegramJoin(user) {
            debug("telegram in join", user);
            if (this._options.oneConnectionByUser) {
                this._getIrcUserChan(user);
            } else {
                var msg = "[Telegram] " + user.username + " join";
                this._ircChannel.sendMessage(msg);
            }
        }

        /**
         * Handle telegram user left message
         * @param {object} user Telegram user data
         */

    }, {
        key: "_handleTelegramLeft",
        value: function _handleTelegramLeft(user) {
            debug("telegram left", user);
            if (this._options.oneConnectionByUser) {
                if (this._haveIrcUser(user)) {
                    var chan = this._getIrcUserChan(user);
                    chan.destroy();
                }
            } else {
                var msg = "[Telegram] " + user.username + " left";
                this._ircChannel.sendMessage(msg);
            }
        }

        /**
         * Bind all events
         * @private
         */

    }, {
        key: "bind",
        value: function bind() {
            var _this3 = this;

            // IRC

            this._ircConnector.waitForRegistered().then(function () {
                _this3._ircSuccessRegistered = true;
                _this3.emit("irc:registered");
            });

            this._ircChannel.on("message", this._handlers.ircMessage);

            this._ircChannel.on("action", this._handlers.ircAction);

            this._ircChannel.on("join", this._handlers.ircJoin);

            this._ircChannel.on("left", this._handlers.ircLeft);

            this._ircChannel.on("topic", this._handlers.ircTopic);

            // Telegram

            this._telegramConnector.me.then(function (me) {
                _this3._telegramSuccessMe = true;
                _this3.emit("telegram:me", me);
            });

            this._telegramChannel.on("message", this._handlers.telegramMessage);

            this._telegramChannel.on("join", this._handlers.telegramJoin);

            this._telegramChannel.on("left", this._handlers.telegramLeft);
        }
    }]);

    return Bridge;
}(_events.EventEmitter);

exports.default = Bridge;