"use strict";

/**
 * @TODO: Add expire time for channel info storage 
 */

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.getBridgeConfig = exports.config = exports.bridges = exports.deleteBridgeConfig = exports.saveBridgeConfig = exports.saveConfig = exports.renderConfigFile = exports.checkDataDir = exports.checkConfigDir = exports.createDataDir = exports.createConfigDir = exports.createDir = undefined;

var _package = require("../package.json");

var _package2 = _interopRequireDefault(_package);

var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _os = require("os");

var _etc = require("etc");

var _etc2 = _interopRequireDefault(_etc);

var _etcYaml = require("etc-yaml");

var _etcYaml2 = _interopRequireDefault(_etcYaml);

var _prettyjson = require("prettyjson");

var _prettyjson2 = _interopRequireDefault(_prettyjson);

var _lodash = require("lodash");

var _debug = require("debug");

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var debug = (0, _debug2.default)("config");

var UID = process.getuid();

var etcDir = void 0;
var varLibDir = void 0;
var dataDir = void 0;
var configDir = void 0;
var homeDir = void 0;

if (process.platform === 'win32') {
    varLibDir = process.env['ALLUSERSAPPDATA'];
    etcDir = varLibDir;
} else {
    // Asume unix
    etcDir = '/etc';
    varLibDir = '/var/lib';
}

if (UID === 0) {
    configDir = _path2.default.join(etcDir, _package2.default.name);
    dataDir = _path2.default.join(varLibDir, _package2.default.name);
} else {
    if (process.versions.node.match(/^6\./)) {
        if (process.platform === 'win32') {
            homeDir = process.env['USERPROFILE'] || process.env['HOMEPATH'];
        } else {
            homeDir = process.env['HOME'];
        }
    } else if (process.versions.node.match(/^7\./)) {
        var user = (0, _os.userInfo)();
        homeDir = user.homedir;
    } else {
        throw new Error('Incompatible node version');
    }

    configDir = _path2.default.join(homeDir, "." + _package2.default.name);
    dataDir = _path2.default.join(homeDir, "." + _package2.default.name);
}

if (process.env[_package2.default.name.toUpperCase() + "_CONFIG_DIR"]) {
    configDir = process.env[_package2.default.name.toUpperCase() + "_CONFIG_DIR"];
}

if (process.env[_package2.default.name.toUpperCase() + "_DATA_DIR"]) {
    dataDir = process.env[_package2.default.name.toUpperCase() + "_DATA_DIR"];
}

var confpath = _path2.default.join(configDir, "config.yml");
var bridgespath = _path2.default.join(configDir, "bridges");
var pluginspath = _path2.default.join(configDir, "plugins");

var createDir = exports.createDir = function createDir(dir) {
    _fs2.default.mkdirSync(dir, 448);
};

var createConfigDir = exports.createConfigDir = function createConfigDir() {
    debug("Creating config directory in " + configDir);
    createDir(configDir);
    createDir(bridgespath);
    createDir(pluginspath);
};

var createDataDir = exports.createDataDir = function createDataDir() {
    debug("Creating data directory in " + dataDir);
    return createDir(dataDir);
};

var checkConfigDir = exports.checkConfigDir = function checkConfigDir() {
    var created = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;

    var stats = void 0;
    debug("Check config directory " + configDir);

    try {
        stats = _fs2.default.lstatSync(configDir);
    } catch (e) {
        debug("Does not exist");
        if (created) {
            return createConfigDir();
        }

        return false;
    }

    if (!stats.isDirectory(configDir)) {
        throw new Error(configDir + " is not a directory");
    }

    return true;
};

var checkDataDir = exports.checkDataDir = function checkDataDir() {
    var created = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;

    var stats = void 0;
    debug("Check data directory " + dataDir);

    try {
        stats = _fs2.default.lstatSync(dataDir);
    } catch (e) {
        debug("Does not exist");
        if (created) {
            return createDataDir();
        }

        return false;
    }

    if (!stats.isDirectory(dataDir)) {
        throw new Error(dataDir + " is not a directory");
    }

    return true;
};

var renderConfigFile = exports.renderConfigFile = function renderConfigFile() {
    var configJson = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
    var final = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;


    if (!configJson) {
        configJson = config.toJSON();
    }

    delete configJson.bridges;
    delete configJson.plugins;

    return _prettyjson2.default.render(configJson, {
        noColor: final,
        indent: 2
    });
};

var saveConfig = exports.saveConfig = function saveConfig() {
    var data = new Buffer(renderConfigFile(null, true) + "\n", 'utf8');
    return _fs2.default.writeFileSync(confpath, data);
};

var saveBridgeConfig = exports.saveBridgeConfig = function saveBridgeConfig(bdata) {
    var data = new Buffer(renderConfigFile(bdata, true) + "\n", 'utf8');
    var file = _path2.default.join(bridgespath, bdata.name + ".yml");
    return _fs2.default.writeFileSync(file, data);
};

var deleteBridgeConfig = exports.deleteBridgeConfig = function deleteBridgeConfig(name) {
    var file = _path2.default.join(bridgespath, name + ".yml");
    _fs2.default.unlinkSync(file);
};

var bridges = exports.bridges = (0, _etc2.default)().use(_etcYaml2.default).folder(bridgespath);

var config = exports.config = (0, _etc2.default)().use(_etcYaml2.default).file(confpath).add({
    bridges: (0, _lodash.values)(bridges.toJSON())
}).add({
    channelsdb: _path2.default.join(dataDir, "channels.dat"),
    user: 'nobody',
    group: 'nobody'
});

exports.default = config;
var getBridgeConfig = exports.getBridgeConfig = function getBridgeConfig(name) {
    var bridgeList = config.get("bridges") || [];
    var bridge = bridgeList.find(function (x) {
        return x.name === name;
    });

    debug("Preparing config for bridge " + name);

    if (!bridge) {
        throw new Error("Bridge does not exists");
    }

    bridge = (0, _lodash.assignIn)({}, {
        enable: true,
        prefix: config.get("prefix"),
        suffix: config.get("suffix"),
        oneConnectionByUser: config.get("oneConnectionByUser"),
        showJoinLeft: config.get("showJoinLeft"),
        ircScapeCharacter: config.get("ircScapeCharacter")
    }, bridge);

    for (var i in bridge) {
        if (typeof bridge[i] === "undefined") {
            delete bridge[i];
        }
    }

    var ircConfig = config.get("irc");
    var telegramConfig = config.get("telegram");

    bridge.irc = (0, _lodash.assignIn)({}, ircConfig, bridge.irc || {});
    bridge.telegram = (0, _lodash.assignIn)({}, telegramConfig, bridge.telegram || {});

    return bridge;
};