"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.IRCChannel = exports.defaultConfig = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _debug = require("debug");

var _debug2 = _interopRequireDefault(_debug);

var _events = require("events");

var _lodash = require("lodash");

var _irc = require("irc");

var _config = require("./config");

var _config2 = _interopRequireDefault(_config);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Promise = require("bluebird");

var instances = [];

var debug = {
    irc: (0, _debug2.default)("irc")
};

var defaultConfig = exports.defaultConfig = (0, _lodash.assignIn)({
    server: null,
    nick: null,
    port: 6697,
    ssl: true,
    autoConnect: true,
    channels: null
}, _config2.default.get("ircDefaults") || {});

/**
 * An IRCChannel
 */

var IRCChannel = exports.IRCChannel = function (_EventEmitter) {
    _inherits(IRCChannel, _EventEmitter);

    /**
     * Create new channel handler
     * @param {string} channel The channel name
     * @param {IRCConnection} connection The connection father
     */
    function IRCChannel(channel, connection) {
        _classCallCheck(this, IRCChannel);

        var _this = _possibleConstructorReturn(this, (IRCChannel.__proto__ || Object.getPrototypeOf(IRCChannel)).call(this));

        debug.irc("Creating IRCChannel for " + channel + " for " + connection.ident);
        _this._channel = channel;
        _this._connection = connection;
        _this._nicks = [connection.nick];

        _this._handlers = {
            message: _this._handleMessage.bind(_this),
            join: _this._handleJoin.bind(_this),
            left: _this._handleLeft.bind(_this),
            topic: _this._handleTopic.bind(_this),
            action: _this._handleAction.bind(_this),
            changeNick: _this._handleChangeNick.bind(_this)
        };

        _this.bind();

        return _this;
    }

    _createClass(IRCChannel, [{
        key: "_handleMessage",
        value: function _handleMessage(nick, message) {
            if (!message) {
                return;
            }

            this.emit("message", nick, message);
        }
    }, {
        key: "_handleAction",
        value: function _handleAction(nick, message) {
            if (!message) {
                return;
            }

            this.emit("action", nick, message);
        }
    }, {
        key: "_handleJoin",
        value: function _handleJoin(nick) {
            this.emit("join", nick);
        }
    }, {
        key: "_handleLeft",
        value: function _handleLeft(nick) {
            this.emit("left", nick);
        }
    }, {
        key: "_handleTopic",
        value: function _handleTopic(channel, topic, nick, message) {
            this.emit("topic", channel, topic, nick, message);
        }
    }, {
        key: "_handleChangeNick",
        value: function _handleChangeNick(oldNick, newNick) {
            this._nicks = this._nicks.filter(function (n) {
                return n !== oldNick;
            });
            this._nicks.push(newNick);
        }
    }, {
        key: "bind",
        value: function bind() {
            var channel = this._channel;

            this._connection.on(channel + ":join", this._handlers.join);

            this._connection.on(channel + ":part", this._handlers.left);

            this._connection.on(channel + ":topic", this._handlers.topic);

            this._connection.on(channel + ":message", this._handlers.message);

            this._connection.on(channel + ":action", this._handlers.action);

            this._connection.on("changeMasterNick", this._handlers.changeNick);
        }
    }, {
        key: "unbind",
        value: function unbind() {
            var channel = this._channel;

            this._connection.removeListener(channel + ":join", this._handlers.join);

            this._connection.removeListener(channel + ":part", this._handlers.left);

            this._connection.removeListener(channel + ":topic", this._handlers.topic);

            this._connection.removeListener(channel + ":message", this._handlers.message);

            this._connection.removeListener(channel + ":action", this._handlers.action);

            this._connection.removeListener("changeMasterNick", this._handlers.changeNick);
        }
    }, {
        key: "sendMessage",
        value: function sendMessage(msg) {
            return this._connection.sendMessage(this._channel, msg);
        }

        /**
         * Return true if the channel has an own nickname
         * @param {string} nickName The nickname to find
         * @return {bool}
         */

    }, {
        key: "hasNick",
        value: function hasNick(nickName) {
            return this._nicks.indexOf(nickName) !== -1;
        }
    }, {
        key: "destroy",
        value: function destroy() {
            debug.irc("Destroy channel " + this.name);
            this.unbind();
            this._connection.removeChannel(this.name);
        }

        /**
         * The name of the channel
         * @property name
         */

    }, {
        key: "name",
        get: function get() {
            return "" + this._channel;
        }
    }]);

    return IRCChannel;
}(_events.EventEmitter);

/**
 * IRC connection
 */


var IRCConnection = function (_EventEmitter2) {
    _inherits(IRCConnection, _EventEmitter2);

    /**
     * Create new connection
     * @param {object} options  The options for connection
     * @param {string} options.server Server ip or name
     * @param {string} options.nick The nick in the server
     * @param {number} [options.port] Port number, by default 6697
     * @param {boolean} [options.ssl] Use SSL, by default true
     * @param {boolean} [options.autoConnect] Auto-connect to server, by
     *                                        default true
     * @param {Array:<string>} [options.channels] Channels to subscribe
     */
    function IRCConnection(options) {
        _classCallCheck(this, IRCConnection);

        var _this2 = _possibleConstructorReturn(this, (IRCConnection.__proto__ || Object.getPrototypeOf(IRCConnection)).call(this));

        _this2._options = (0, _lodash.assignIn)({}, defaultConfig, options);
        _this2._client = null;
        _this2._channels = [];
        _this2._registered = false;
        _this2._originalNick = _this2._options.nick;

        debug.irc("Start irc connection " + _this2._options.server);

        if (!_this2._options.server || !_this2._options.port || !_this2._options.nick) {
            throw new Error("You need to specify some server and nick");
        }

        if (options.channels) {
            options.channels.forEach(function (chan) {
                _this2.addChannel(chan);
            });
        }

        delete _this2._options.channels;

        if (_this2._options.autoConnect) {
            (0, _lodash.noop)(_this2.client);
        }

        instances.push(_this2);

        return _this2;
    }

    /**
     * Handle join events from the server
     * @param {string} channel
     * @param {string} nick
     */


    _createClass(IRCConnection, [{
        key: "_handleJoin",
        value: function _handleJoin(channel, nick) {
            var ownChannel = this.getChannel(channel);

            if (ownChannel) {
                this.emit("join", channel, nick);
                this.emit(channel + ":join", nick);
            } else {
                debug.irc(nick + " join to unhandled channel " + channel);
            }
        }

        /**
         * Handle incomming message from the server
         * @param {string} from
         * @param {string} to
         * @param {string} message
         */

    }, {
        key: "_handleIncommingMessage",
        value: function _handleIncommingMessage(from, to, message) {
            var ownChannel = this.getChannel(to);

            if (ownChannel && !ownChannel.hasNick(from)) {
                this.emit(ownChannel.name + ":message", from, message);
            }
        }

        /**
         * Handle incomming action from the server
         * @param {string} from
         * @param {string} to
         * @param {string} message
         */

    }, {
        key: "_handleIncommingAction",
        value: function _handleIncommingAction(from, to, message) {
            var ownChannel = this.getChannel(to);

            if (ownChannel && !ownChannel.hasNick(from)) {
                this.emit(ownChannel.name + ":action", from, message);
            }
        }

        /**
         * Handle part from the server
         * @param {string} channel
         * @param {string} nick
         * @param {string} message
         */

    }, {
        key: "_handlePart",
        value: function _handlePart(channel, nick, message) {
            var ownChannel = this.getChannel(channel);

            if (ownChannel && !ownChannel.hasNick(nick)) {
                this.emit(ownChannel.name + ":part", nick, message);
            }
        }

        /**
         * Handle topic from the server
         * @param {string} channel
         * @param {string} topic
         * @param {string} nick
         * @param {string} message
         */

    }, {
        key: "_handleTopic",
        value: function _handleTopic(channel, topic, nick, message) {
            var ownChannel = this.getChannel(channel);

            if (ownChannel && !ownChannel.hasNick(nick)) {
                this.emit(ownChannel.name + ":topic", channel, topic, nick, message);
            }
        }
    }, {
        key: "waitForRegistered",
        value: function waitForRegistered() {
            var _this3 = this;

            if (this._registered) {
                return Promise.resolve();
            }

            return new Promise(function (resolve) {
                _this3.once("irc:registered", resolve);
            });
        }

        /**
         * Get a channel handler from name
         * @param {string} channelName Channel name
         * @return {null|IRCChannel}
         */

    }, {
        key: "getChannel",
        value: function getChannel(channelName) {
            return this._channels.find(function (c) {
                return c.name === channelName;
            });
        }

        /**
         * Add channel handle
         * @param {string} name
         * @return {Promise<IRCChannel>}
         */

    }, {
        key: "addChannel",
        value: function addChannel(channelName) {
            var _this4 = this;

            debug.irc("Add channel to IRCConnection " + channelName);

            if (this._channels.find(function (c) {
                return c.name === channelName;
            })) {
                throw new Error("Channel already exists");
            }

            var channel = new IRCChannel(channelName, this);
            this._channels.push(channel);

            this.waitForRegistered().then(function () {
                debug.irc("Send irc join");
                _this4.client.join(channelName);
            });

            return channel;
        }
    }, {
        key: "removeChannel",
        value: function removeChannel(channelName) {
            var _this5 = this;

            var channel = this._channels.find(function (c) {
                return c.name === channelName;
            });

            if (!channel) {
                throw new Error("Channel does not exists");
            }

            this.waitForRegistered().then(function () {
                _this5.client.part(channelName);
            });

            this._channels = this._channels.filter(function (x) {
                return x.name !== channel.name;
            });
        }

        /**
         * Client object
         * @property
         * @see https://github.com/martynsmith/node-irc
         */

    }, {
        key: "sendMessage",
        value: function sendMessage(channel, msg) {
            var _this6 = this;

            // hack ¿?
            (0, _lodash.noop)(this.client);
            this.waitForRegistered().then(function () {
                _this6.client.say(channel, msg);
            });
        }

        /**
         * Nick
         * @property
         */

    }, {
        key: "client",
        get: function get() {
            var _this7 = this;

            if (this._client) {
                return this._client;
            }

            var _options = this._options,
                server = _options.server,
                port = _options.port,
                ssl = _options.ssl,
                nick = _options.nick,
                autoConnect = _options.autoConnect,
                channels = _options.channels;


            this._client = new _irc.Client(server, nick, {
                port: port,
                autoConnect: autoConnect,
                channels: channels,
                secure: ssl
            });

            this._client.addListener("message", function (from, to, message) {
                debug.irc(from + " => " + to + ": " + message);
                _this7.emit("irc:message", from, to, message);
                return _this7._handleIncommingMessage(from, to, message);
            });

            this._client.addListener("action", function (from, to, message) {
                debug.irc(from + " => " + to + ": " + message);
                _this7.emit("irc:action", from, to, message);
                return _this7._handleIncommingAction(from, to, message);
            });

            this._client.addListener("join", function (channel, nick, message) {
                debug.irc(nick + " join in " + channel);
                _this7.emit("irc:join", channel, nick, message);
                return _this7._handleJoin(channel, nick, message);
            });

            this._client.addListener("part", function (channel, nick, message) {
                debug.irc(nick + " part of " + channel);
                _this7.emit("irc:part", channel, nick, message);
                return _this7._handlePart(channel, nick, message);
            });

            this._client.addListener("topic", function (channel, topic, nick, message) {
                debug.irc(nick + " changed the topic of " + channel + " to " + topic);
                _this7.emit("irc:topic", channel, topic, nick, message);
                return _this7._handleTopic(channel, topic, nick, message);
            });

            this._client.addListener("registered", function (data) {
                var nick = data.args[0];
                debug.irc(nick + " registered");

                _this7._registered = true;

                _this7.emit("irc:registered", nick, data);
                var oldNick = _this7._options.nick;
                _this7._options.nick = nick;
                _this7.emit("changeMasterNick", oldNick, nick);
            });

            this._client.addListener("error", function (message) {
                debug.irc("error: ", message);
                _this7.emit("error", message);
            });

            this._ownUsernames = [this._options.nick];

            return this._client;
        }
    }, {
        key: "nick",
        get: function get() {
            return this._options.nick;
        }

        /**
         * identifier
         * @property
         */

    }, {
        key: "identifier",
        get: function get() {
            var _options2 = this._options,
                port = _options2.port,
                server = _options2.server,
                ssl = _options2.ssl;

            var originalNick = this._originalNick;
            return originalNick + "@" + server + ":" + port + (ssl ? "+" : "");
        }

        /**
         * Get connection by server options, if does not exists, create one
         * @param {object} options @see IRCConnection.contrusctor
         * @return {IRCConnection}
         */

    }], [{
        key: "getByServerOptions",
        value: function getByServerOptions(uoptions) {
            var options = (0, _lodash.assignIn)({}, defaultConfig, uoptions);
            var nick = options.nick,
                port = options.port,
                server = options.server,
                ssl = options.ssl;

            var identifier = nick + "@" + server + ":" + port + (ssl ? "+" : "");

            debug.irc("Search server by identifier " + identifier);

            var instance = instances.find(function (i) {
                return i.identifier === identifier;
            });

            if (!instance) {
                instance = new IRCConnection(options);
            }

            return instance;
        }
    }]);

    return IRCConnection;
}(_events.EventEmitter);

exports.default = IRCConnection;