"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _events = require("events");

var _debug = require("debug");

var _debug2 = _interopRequireDefault(_debug);

var _bridge = require("./bridge");

var _bridge2 = _interopRequireDefault(_bridge);

var _config = require("./config");

var _config2 = _interopRequireDefault(_config);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Promise = require("bluebird");

var debug = (0, _debug2.default)("session");

/**
 * Bridges up session
 */

var Session = function (_EventEmitter) {
    _inherits(Session, _EventEmitter);

    /**
     * Create session
     * @param {object} [options] Options
     */
    function Session() {
        var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

        _classCallCheck(this, Session);

        var _this = _possibleConstructorReturn(this, (Session.__proto__ || Object.getPrototypeOf(Session)).call(this));

        _this._options = options;
        _this._bridgesConfig = _this._makeBridgesTree();
        _this._started = false;
        _this._bridges = [];
        return _this;
    }

    _createClass(Session, [{
        key: "_makeBridgesTree",
        value: function _makeBridgesTree() {
            var _this2 = this;

            if (this._options.only) {
                debug("only option for " + this._options.only);
            }

            return (_config2.default.get("bridges") || []).map(function (x) {
                return x.name;
            }).filter(function (b) {
                if (_this2._options.only) {
                    return b === _this2._options.only;
                } else {
                    return true;
                }
            }).map(function (n) {
                return (0, _config.getBridgeConfig)(n);
            }).filter(function (n) {
                return n.enable;
            });
        }
    }, {
        key: "start",
        value: function start() {
            var _this3 = this;

            if (this._started) {
                throw new Error("Already started");
            }

            if (!this._bridgesConfig.length) {
                throw new Error("Nothing to start");
            }

            debug("Starting bridges");

            this._bridges = this._bridgesConfig.map(function (bridgeConfig) {
                var ircChannel = bridgeConfig.irc.channel;
                var telegramChannel = bridgeConfig.telegram.channel;

                if (!ircChannel || !telegramChannel) {
                    throw new Error("Bridge " + bridgeConfig.name + " has a error in " + "IRC or telegram channel definition");
                }

                debug("Starting " + bridgeConfig.name + " " + ircChannel + " <-> " + (" " + telegramChannel));

                var bridge = new _bridge2.default(bridgeConfig.name, ircChannel, telegramChannel, bridgeConfig);

                Promise.all([new Promise(function (resolve) {
                    bridge.once("irc:registered", function () {
                        debug("Bridge " + bridgeConfig.name + " registered with IRC");
                        return resolve();
                    });
                }), new Promise(function (resolve) {
                    bridge.once("telegram:me", function () {
                        debug("Bridge " + bridgeConfig.name + " success with Telegram");
                        return resolve();
                    });
                })]).then(function () {
                    debug("Bridge " + bridgeConfig.name + " stablished");
                    _this3.emit("bridgestablished", bridgeConfig.name);
                });

                return bridge;
            });

            this._started = true;
        }
    }]);

    return Session;
}(_events.EventEmitter);

exports.default = Session;