"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.ChannelsInfo = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _debug = require("debug");

var _debug2 = _interopRequireDefault(_debug);

var _nodeStorage = require("node-storage");

var _nodeStorage2 = _interopRequireDefault(_nodeStorage);

var _config = require("./config");

var _config2 = _interopRequireDefault(_config);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var CHANNELS_DB_PATH = _path2.default.normalize(_config2.default.get("channelsdb"));

var debug = {
    channelsInfo: (0, _debug2.default)("channels-info")
};

var instance = null;

var ChannelsInfo = exports.ChannelsInfo = function () {
    function ChannelsInfo() {
        _classCallCheck(this, ChannelsInfo);

        ChannelsInfo.checkDataDir();
        this._storage = new _nodeStorage2.default(CHANNELS_DB_PATH);
        this._data = null;
        this._load();
    }

    _createClass(ChannelsInfo, [{
        key: "_load",
        value: function _load() {
            debug.channelsInfo("Load channels stored information");
            var data = this._storage.get("channels");

            if (!data) {
                debug.channelsInfo("There are nothing, empty");
                data = [];
            }

            this._data = data;
        }
    }, {
        key: "sync",
        value: function sync() {
            this._storage.put("channels", this._data);
        }
    }, {
        key: "remove",
        value: function remove(channel) {
            var sync = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

            var channelId = typeof channel === "number" ? channel : channel.id;
            debug.channelsInfo("Remove channel info for " + channelId);

            this._data = this._data.filter(function (x) {
                return x.id !== channelId;
            });

            if (sync) {
                this.sync();
            }
        }
    }, {
        key: "save",
        value: function save(channel) {
            var sync = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

            debug.channelsInfo("Save channel info for " + channel.id);

            var actual = this._data.find(function (x) {
                return x.id === channel.id;
            });

            if (actual) {
                debug.channelsInfo("This exists, remove old");
                this.remove(channel, false);
            }

            this._data.push(channel);

            if (sync) {
                this.sync();
            }
        }
    }, {
        key: "list",
        value: function list() {
            return this._data;
        }
    }, {
        key: "clear",
        value: function clear() {
            var sync = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;

            debug.channelsInfo("Clear all data");
            this._data = [];
            if (sync) {
                this.sync();
            }
        }
    }, {
        key: "each",
        value: function each(cb) {
            return this._data.forEach(cb);
        }
    }, {
        key: "sort",
        value: function sort(cb) {
            return this._data.sort(cb);
        }
    }, {
        key: "map",
        value: function map(cb) {
            return this._data.map(cb);
        }
    }, {
        key: "find",
        value: function find(cond) {
            debug.channelsInfo("Find info for " + cond);

            if (typeof cond === "number") {
                return this._data.find(function (x) {
                    return x.id === cond;
                });
            } else {
                return this._data.find(function (x) {
                    return x.title === cond;
                });
            }
        }
    }], [{
        key: "createDataDir",
        value: function createDataDir(dirname) {
            debug.channelsInfo("Creating data directory in " + dirname);
            _fs2.default.mkdirSync(dirname, 448);
        }
    }, {
        key: "checkDataDir",
        value: function checkDataDir() {
            var stats = void 0,
                directory = _path2.default.dirname(CHANNELS_DB_PATH);
            debug.channelsInfo("Check for data directory in " + directory);

            try {
                stats = _fs2.default.lstatSync(directory);
            } catch (e) {
                debug.channelsInfo("Don't exists");
                return ChannelsInfo.createDataDir(directory);
            }

            if (!stats.isDirectory(directory)) {
                throw new Error(directory + " is not a directory");
            }
        }
    }, {
        key: "getInstance",
        value: function getInstance() {
            if (!instance) {
                instance = new ChannelsInfo();
            }

            return instance;
        }
    }]);

    return ChannelsInfo;
}();