"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.TelegramChannel = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _events = require("events");

var _debug = require("debug");

var _debug2 = _interopRequireDefault(_debug);

var _lodash = require("lodash");

var _nodeTelegramBotApi = require("node-telegram-bot-api");

var _nodeTelegramBotApi2 = _interopRequireDefault(_nodeTelegramBotApi);

var _storage = require("./storage");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Promise = require("bluebird");

var debug = {
    channel: (0, _debug2.default)("telegram-channel"),
    connection: (0, _debug2.default)("telegram-connection")
};

var NEW_CHAT_PARTICIPANT = "new_chat_participant";
var LEFT_CHAT_PARTICIPANT = "left_chat_participant";
var CHANEXP = /^-?[0-9]+$/;

var defaultConfig = {
    token: null,
    autoConnect: true
};

var instances = [];

/**
 * Telegram channel wrapper
 */

var TelegramChannel = exports.TelegramChannel = function (_EventEmitter) {
    _inherits(TelegramChannel, _EventEmitter);

    /**
     * Create channel wrapper for telegram
     * @param {int|string} channel The channel title, or ID
     * @param {TelegramConnection} connector The connector instance
     */
    function TelegramChannel(channel, connector) {
        _classCallCheck(this, TelegramChannel);

        var _this = _possibleConstructorReturn(this, (TelegramChannel.__proto__ || Object.getPrototypeOf(TelegramChannel)).call(this));

        debug.channel("Start channel");
        _this._channel = channel;
        _this._connector = connector;

        if (typeof _this._channel === 'string' && _this._channel.match(CHANEXP)) {
            debug.channel("Channel is string but has format of ID");
            _this._channel = parseInt(_this._channel, 10);
        }

        _this._hasInfo = false;
        _this._chatId = null;
        _this._chatType = null;
        _this._chatTitle = null;
        _this._chatLastUpdated = null;

        var channelsInfo = _storage.ChannelsInfo.getInstance();
        var chatInfo = channelsInfo.find(_this._channel);

        _this._handlers = {
            message: _this._handleMessage.bind(_this),
            join: _this._handleNewParticipant.bind(_this),
            left: _this._handleLeftParticipant.bind(_this)
        };

        if (chatInfo) {
            debug.channel("Have info about channel, setting");
            _this.setData(chatInfo, false);
        } else {
            debug.channel("Don't have info about channel, bind");
            _this.bind();
        }

        return _this;
    }

    _createClass(TelegramChannel, [{
        key: "_handleMessage",
        value: function _handleMessage(user, message) {
            if (!message) {
                debug.channel("unknow message, ignoring");
                return;
            }
            this.emit("message", user, message);
        }
    }, {
        key: "_handleNewParticipant",
        value: function _handleNewParticipant() {
            for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
                args[_key] = arguments[_key];
            }

            this.emit.apply(this, ["join"].concat(args));
        }
    }, {
        key: "_handleLeftParticipant",
        value: function _handleLeftParticipant() {
            for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
                args[_key2] = arguments[_key2];
            }

            this.emit.apply(this, ["left"].concat(args));
        }
    }, {
        key: "_handleInfoUpdate",
        value: function _handleInfoUpdate(data) {
            if (data.id === this._channel || data.title === this._channel) {
                this.setData(data);
            }
        }
    }, {
        key: "unbind",
        value: function unbind() {
            var prefix = this.channelPrefix;

            debug.channel("unbind channel " + prefix);

            this._connector.removeListener(prefix + ":message", this._handlers.message);

            this._connector.removeListener(prefix + ":newparticipant", this._handleNewParticipant);

            this._connector.removeListener(prefix + ":leftparticipant", this._handleLeftParticipant);

            this._connector.removeListener("chatinformationupdate", this._handleInfoUpdate);
        }
    }, {
        key: "bind",
        value: function bind() {

            var prefix = this.channelPrefix;

            debug.channel("bind channel " + prefix);

            this._connector.on(prefix + ":message", this._handlers.message);

            this._connector.on(prefix + ":newparticipant", this._handleNewParticipant.bind(this));

            this._connector.on(prefix + ":leftparticipant", this._handleLeftParticipant.bind(this));

            this._connector.on("chatinformationupdate", this._handleInfoUpdate.bind(this));
        }

        /**
         * Set channel information
         * @param {object} data
         */

    }, {
        key: "setData",
        value: function setData(data) {
            var sync = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;


            this.unbind();

            if (sync && (this._chatTitle !== data.title || this._chatType !== data.type)) {
                _storage.ChannelsInfo.getInstance().save(data);
            }

            this._hasInfo = true;
            this._chatId = data.id;
            this._chatType = data.type;
            this._chatTitle = data.title;
            this._chatLastUpdated = data.updated;

            this.bind();

            debug.channel("Information updateted");
            this.emit("updateinformation");
        }

        /**
         * Wait to get channel information, this is necesary for exampleto
         * send a message
         * @return {Promise}
         */

    }, {
        key: "waitInformation",
        value: function waitInformation() {
            var _this2 = this;

            if (this._hasInfo) {
                return Promise.resolve();
            }

            return new Promise(function (resolve) {
                _this2.once("updateinformation", function () {
                    return resolve();
                });
            });
        }

        /**
         * Send a message to the channel
         * @param {string} message
         */

    }, {
        key: "sendMessage",
        value: function sendMessage(message, options) {
            var _this3 = this;

            this.waitInformation().then(function () {
                return _this3._connector.send(_this3._chatId, message, options || {});
            });
        }

        /**
         * Get the channel prefix
         * @return {string} The channel prefix
         */

    }, {
        key: "channelPrefix",
        get: function get() {

            if (this._hasInfo) {
                return "#" + this._chatId;
            } else if (typeof this._channel === "string") {
                return "@@" + this._channel;
            } else if (typeof this._channel === "number") {
                return "#" + this._channel;
            } else {
                throw new Error("Unknow channel type");
            }
        }

        /**
         * Name of channel
         * @property name
         */

    }, {
        key: "name",
        get: function get() {
            return this.channelPrefix;
        }
    }]);

    return TelegramChannel;
}(_events.EventEmitter);

/**
 * Telegram connection helper
 */


var TelegramConnection = function (_EventEmitter2) {
    _inherits(TelegramConnection, _EventEmitter2);

    /**
     * Create new connection with telegram
     * @param {object} options  Options
     * @param {string} options.token Telegram API token
     */
    function TelegramConnection(options) {
        _classCallCheck(this, TelegramConnection);

        var _this4 = _possibleConstructorReturn(this, (TelegramConnection.__proto__ || Object.getPrototypeOf(TelegramConnection)).call(this));

        _this4._tgBot = null;
        _this4._options = (0, _lodash.assignIn)({}, defaultConfig, options);
        _this4._channels = [];

        if (_this4._options.autoConnect) {
            (0, _lodash.noop)(_this4.tgBot);
        }

        instances.push(_this4);

        return _this4;
    }

    /**
     * Send a message
     * @param {string|number} chatId The chat ID
     * @param {string} msg Message
     */


    _createClass(TelegramConnection, [{
        key: "send",
        value: function send(chatId, msg, options) {
            return this.tgBot.sendMessage(chatId, msg, options);
        }

        /**
         * Get the chat info from a incomming message
         * @param {object} msg
         * @return {object}
         */

    }, {
        key: "getChatInfo",
        value: function getChatInfo(msg) {
            var chat = msg.chat;

            return {
                chatId: chat.id,
                chatType: chat.type,
                chatTitle: chat.title
            };
        }

        /**
         * Generate a chat event prefix
         * @param {object} msg
         * @return {object}
         */

    }, {
        key: "getChatEvPrefix",
        value: function getChatEvPrefix(msg) {
            var _getChatInfo = this.getChatInfo(msg),
                chatId = _getChatInfo.chatId,
                chatTitle = _getChatInfo.chatTitle;

            return {
                idPrefix: "#" + chatId,
                titlePrefix: "@@" + chatTitle
            };
        }

        /**
         * Refresh the chat information when it's received a new message
         * @param {object} chatData
         */

    }, {
        key: "refreshChatInformation",
        value: function refreshChatInformation(chatData) {
            var id = chatData.id,
                title = chatData.title,
                type = chatData.type;

            var now = new Date().getTime();
            var channelsInfo = _storage.ChannelsInfo.getInstance();

            var current = channelsInfo.find(id);

            if (current && current.type === type && current.id === id && current.title === title) {
                return;
            }

            var data = {
                id: id,
                title: title,
                type: type,
                updated: now
            };

            channelsInfo.save(data);
            debug.connection("channel info update", data.id, data.title);
            this.emit("chatinformationupdate", data);
        }

        /**
         * Follow a channel
         * @param {string|number} channelId The channel to follow
         * @return {TelegramChannel} The channel wrapper
         */

    }, {
        key: "followChannel",
        value: function followChannel(channelId) {
            var channel = new TelegramChannel(channelId, this);
            this._channels.push(channel);
            return channel;
        }

        /**
         * TelegramBot connection
         * @see https://github.com/yagop/node-telegram-bot-api
         * @property tgBot
         */

    }, {
        key: "tgBot",
        get: function get() {
            var _this5 = this;

            if (this._tgBot) {
                return this._tgBot;
            }

            debug.connection("Token", this._options.token);
            this._tgBot = new _nodeTelegramBotApi2.default(this._options.token, {
                polling: true
            });

            this._tgBot.on("message", function (msg) {
                var from = msg.from,
                    text = msg.text,
                    chat = msg.chat;

                var _getChatEvPrefix = _this5.getChatEvPrefix(msg),
                    idPrefix = _getChatEvPrefix.idPrefix,
                    titlePrefix = _getChatEvPrefix.titlePrefix;

                _this5.refreshChatInformation(chat);
                debug.connection(titlePrefix + " (" + idPrefix + "): " + from.id + " -> " + text);
                _this5.emit("telegram:message", chat, from, text);
                _this5.emit(idPrefix + ":message", from, text, chat);
                _this5.emit(titlePrefix + ":message", from, text, chat);
            });

            this._tgBot.on("new_chat_participant", function (msg) {
                var newParticipant = msg[NEW_CHAT_PARTICIPANT];

                var _getChatEvPrefix2 = _this5.getChatEvPrefix(msg),
                    idPrefix = _getChatEvPrefix2.idPrefix,
                    titlePrefix = _getChatEvPrefix2.titlePrefix;

                _this5.refreshChatInformation(msg.chat);
                debug.connection(titlePrefix + " (" + idPrefix + "): " + ("new participant " + newParticipant));
                _this5.emit("telegram:newparticipant", msg.chat, newParticipant);
                _this5.emit(idPrefix + ":newparticipant", newParticipant);
                _this5.emit(titlePrefix + ":newparticipant", newParticipant);
            });

            this._tgBot.on("left_chat_participant", function (msg) {
                var _getChatEvPrefix3 = _this5.getChatEvPrefix(msg),
                    idPrefix = _getChatEvPrefix3.idPrefix,
                    titlePrefix = _getChatEvPrefix3.titlePrefix;

                var leftParticipant = msg[LEFT_CHAT_PARTICIPANT];
                _this5.refreshChatInformation(msg.chat);
                debug.connection(titlePrefix + " (" + idPrefix + "): " + ("left participant " + leftParticipant));
                _this5.emit("telegram:leftparticipant", msg.chat, leftParticipant);
                _this5.emit(idPrefix + ":leftparticipant", leftParticipant);
                _this5.emit(titlePrefix + ":leftparticipant", leftParticipant);
            });

            this._tgBot.getMe().then(function (me) {
                _this5._me = me;
                _this5.emit("getme");
            });

            return this._tgBot;
        }

        /**
         * Information about the connected bot
         * @property
         */

    }, {
        key: "me",
        get: function get() {
            var _this6 = this;

            if (this._me) {
                return Promise.resolve(this._me);
            }

            return new Promise(function (resolve) {
                _this6.once("getme", function () {
                    return resolve(_this6._me);
                });
            });
        }

        /**
         * The token that is connected the bot
         * @property
         */

    }, {
        key: "token",
        get: function get() {
            return this._options.token;
        }

        /**
         * Get an instance of TelegramConnector by token, if the instance
         * does not exists, it's created, like getInstance, but by token
         * @param {string} token The token
         * @param {object} options Options for new instance creation
         * @return {TelegramConnection}
         */

    }], [{
        key: "getByToken",
        value: function getByToken(token) {
            var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};


            var instance = instances.find(function (i) {
                return i.token === token;
            });

            if (!instance) {
                instance = new TelegramConnection((0, _lodash.assignIn)(options, { token: token }));
            }

            return instance;
        }
    }]);

    return TelegramConnection;
}(_events.EventEmitter);

exports.default = TelegramConnection;